@@ -1,11 +1,11 @@
 use strict;
 use warnings;
-package OpenAPI::Modern; # git description: v0.029-2-gd6e0964
+package OpenAPI::Modern; # git description: v0.030-4-g3022412
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Validate HTTP requests and responses against an OpenAPI document
 # KEYWORDS: validation evaluation JSON Schema OpenAPI Swagger HTTP request response
 
-our $VERSION = '0.030';
+our $VERSION = '0.031';
 
 use 5.020;
 use Moo;
@@ -534,6 +534,8 @@ sub _validate_body_content ($self, $state, $content_obj, $message) {
   $state = { %$state, schema_path => jsonp($state->{schema_path}, 'content', $media_type, 'schema') };
   my $result = $self->_evaluate_subschema($content_ref->$*, $schema, $state);
 
+  return 1 if not is_ref($result);  # schema is an empty hash or boolean true
+
   my $type = (split('/', $state->{data_path}, 3))[1];
   my $keyword = $type eq 'request' ? 'readOnly' : $type eq 'response' ? 'writeOnly' : die "unknown type $type";
 
@@ -580,6 +582,23 @@ sub _resolve_ref ($self, $ref, $state) {
 sub _evaluate_subschema ($self, $data, $schema, $state) {
   return 1 if is_plain_hashref($schema) ? !keys(%$schema) : $schema; # true schema
 
+  if (is_plain_hashref($schema)) {
+    return 1 if !keys(%$schema);
+  }
+  else {
+    return 1 if $schema;
+
+    my @location = unjsonp($state->{data_path});
+    my $location =
+        $location[-1] eq 'body' ? join(' ', @location[-2..-1])
+      : $location[-2] eq 'query' ? 'query parameter'
+      : $location[-2] eq 'path' ? 'path parameter'  # this should never happen
+      : $location[-2] eq 'header' ? join(' ', @location[-3..-2])
+      : $location[-2];  # cookie
+    return E($state, '%s not permitted', $location);
+    return E($state, 'thingy not permitted');
+  }
+
   # treat numeric-looking data as a string, unless "type" explicitly requests number or integer.
   if (is_plain_hashref($schema) and exists $schema->{type} and not is_plain_arrayref($schema->{type})
       and grep $schema->{type} eq $_, qw(number integer) and looks_like_number($data)) {
@@ -675,7 +694,7 @@ OpenAPI::Modern - Validate HTTP requests and responses against an OpenAPI docume
 
 =head1 VERSION
 
-version 0.030
+version 0.031
 
 =head1 SYNOPSIS
 
